@@ -1,3 +1,4 @@
+import functools
 import secrets
 import uuid
 from datetime import date
@@ -139,7 +140,7 @@ class UserProfile(guardian.mixins.GuardianUserMixin, PermissionsMixin, AbstractB
         shift_ids = self.localparticipation_set.filter(
             state__in=with_participation_state_in
         ).values_list("shift", flat=True)
-        return Shift.objects.filter(pk__in=shift_ids)
+        return Shift.objects.filter(pk__in=shift_ids).select_related("event")
 
     def get_workhour_items(self):
         from ephios.core.models import AbstractParticipation
@@ -319,3 +320,33 @@ class WorkingHours(Model):
 
     class Meta:
         db_table = "workinghours"
+
+
+class Notification(Model):
+    slug = models.SlugField(max_length=255)
+    user = models.ForeignKey(
+        get_user_model(),
+        on_delete=models.CASCADE,
+        verbose_name=_("affected user"),
+        null=True,
+    )
+    failed = models.BooleanField(default=False)
+    data = models.JSONField(
+        blank=True, default=dict, encoder=CustomJSONEncoder, decoder=CustomJSONDecoder
+    )
+
+    @functools.cached_property
+    def notification_type(self):
+        from ephios.core.notifications.types import notification_type_from_slug
+
+        return notification_type_from_slug(self.slug)
+
+    @property
+    def subject(self):
+        return self.notification_type.get_subject(self)
+
+    def as_plaintext(self):
+        return self.notification_type.as_plaintext(self)
+
+    def as_html(self):
+        return self.notification_type.as_html(self)
