@@ -9,7 +9,7 @@ from django.views.generic.detail import SingleObjectMixin
 from django_select2.forms import ModelSelect2Widget
 
 from ephios.core.models import AbstractParticipation, Shift, UserProfile
-from ephios.extra.permissions import CustomPermissionRequiredMixin
+from ephios.extra.mixins import CustomPermissionRequiredMixin
 
 
 class BaseDispositionParticipationForm(forms.ModelForm):
@@ -134,7 +134,7 @@ class AddUserView(DispositionBaseViewMixin, TemplateResponseMixin, View):
                 start_index=form.cleaned_data["new_index"],
             )
             form = next(filter(lambda form: form.instance.id == instance.id, formset))
-            return self.render_to_response({"form": form})
+            return self.render_to_response({"form": form, "shift": shift})
         raise Http404()
 
 
@@ -147,7 +147,7 @@ class DispositionView(DispositionBaseViewMixin, TemplateView):
         )
         formset = DispositionParticipationFormset(
             self.request.POST or None,
-            queryset=self.object.participations,
+            queryset=self.object.participations.all(),
             prefix="participations",
         )
         return formset
@@ -156,6 +156,22 @@ class DispositionView(DispositionBaseViewMixin, TemplateView):
         formset = self.get_formset()
         if formset.is_valid():
             formset.save()
+
+            for participation, changed_fields in formset.changed_objects:
+                if "state" in changed_fields:
+                    if participation.state == AbstractParticipation.States.CONFIRMED:
+                        from ephios.core.notifications.types import (
+                            ParticipationConfirmedNotification,
+                        )
+
+                        ParticipationConfirmedNotification.send(participation)
+                    elif participation.state == AbstractParticipation.States.RESPONSIBLE_REJECTED:
+                        from ephios.core.notifications.types import (
+                            ParticipationRejectedNotification,
+                        )
+
+                        ParticipationRejectedNotification.send(participation)
+
             self.object.participations.filter(
                 state=AbstractParticipation.States.GETTING_DISPATCHED
             ).delete()
