@@ -1,49 +1,58 @@
+import dataclasses
 import functools
+import logging
 from argparse import Namespace
-from dataclasses import dataclass, field
+from collections import OrderedDict
 from datetime import date
 from typing import List, Optional
 
-import django.dispatch
 from django import forms
 from django.contrib import messages
 from django.contrib.auth import get_user_model
-from django.core.exceptions import ValidationError
+from django.core.exceptions import PermissionDenied, ValidationError
 from django.db import transaction
-from django.db.models import QuerySet
+from django.db.models import Q, QuerySet
 from django.shortcuts import redirect
 from django.template import Context, Template
 from django.template.defaultfilters import yesno
+from django.urls import reverse
 from django.utils import formats, timezone
-from django.utils.functional import cached_property
+from django.utils.functional import SimpleLazyObject, cached_property
+from django.utils.safestring import mark_safe
 from django.utils.translation import gettext_lazy as _
 from django.views import View
 
 from ephios.core.models import AbstractParticipation, LocalParticipation, Qualification, Shift
 from ephios.extra.widgets import CustomSplitDateTimeWidget
 
+from ..signals import participant_from_request, register_signup_methods
 from .disposition import BaseDispositionParticipationForm
 
-register_signup_methods = django.dispatch.Signal()
+logger = logging.getLogger(__name__)
 
 
-def all_signup_methods():
+def installed_signup_methods():
+    for _, methods in register_signup_methods.send_to_all_plugins(None):
+        yield from methods
+
+
+def enabled_signup_methods():
     for _, methods in register_signup_methods.send(None):
         yield from methods
 
 
 def signup_method_from_slug(slug, shift=None):
-    for method in all_signup_methods():
+    for method in installed_signup_methods():
         if method.slug == slug:
             return method(shift)
     raise ValueError(_("Signup Method '{slug}' was not found.").format(slug=slug))
 
 
-@dataclass(frozen=True)
+@dataclasses.dataclass(frozen=True)
 class AbstractParticipant:
     first_name: str
     last_name: str
-    qualifications: QuerySet = field(hash=False)
+    qualifications: QuerySet = dataclasses.field(hash=False)
     date_of_birth: date
     email: Optional[str]  # if set to None, no notifications are sent
 
@@ -61,7 +70,11 @@ class AbstractParticipant:
         """Return the participation object for a shift. Return None if it does not exist."""
         raise NotImplementedError
 
-    @functools.lru_cache(maxsize=1)
+    def all_participations(self):
+        """Return all participations for this participant"""
+        raise NotImplementedError
+
+    @functools.lru_cache
     def collect_all_qualifications(self) -> set:
         """We collect using breadth first search with one query for every layer of inclusion."""
         all_qualifications = set(self.qualifications)
@@ -79,8 +92,18 @@ class AbstractParticipant:
     def has_qualifications(self, qualifications):
         return set(qualifications) <= self.collect_all_qualifications()
 
+    def reverse_signup_action(self, shift):
+        raise NotImplementedError
 
-@dataclass(frozen=True)
+    def reverse_event_detail(self, event):
+        raise NotImplementedError
+
+    @property
+    def icon(self):
+        return mark_safe('<span class="fa fa-user"></span>')
+
+
+@dataclasses.dataclass(frozen=True)
 class LocalUserParticipant(AbstractParticipant):
     user: get_user_model()
 
@@ -93,15 +116,55 @@ class LocalUserParticipant(AbstractParticipant):
         except LocalParticipation.DoesNotExist:
             return None
 
+    def all_participations(self):
+        return LocalParticipation.objects.filter(user=self.user)
+
+    def reverse_signup_action(self, shift):
+        return reverse("core:signup_action", kwargs=dict(pk=shift.pk))
+
+    def reverse_event_detail(self, event):
+        return event.get_absolute_url()
+
 
 class ParticipationError(ValidationError):
     pass
 
 
+def prevent_getting_participant_from_request_user(request):
+    original_user = request.user
+
+    class ProtectedUser(SimpleLazyObject):
+        def as_participant(self):
+            raise Exception("Access of request.user.as_participant in SignupViews is not allowed.")
+
+    request.user = ProtectedUser(lambda: original_user)
+
+
+def get_nonlocal_participant_from_session(request):
+    for _, participant in participant_from_request.send(sender=None, request=request):
+        if participant is not None:
+            return participant
+    raise PermissionDenied
+
+
 class BaseSignupView(View):
+    """
+    This View reacts to the signup or decline buttons being pressed using a POST request.
+    It can be modified to not directly create a participation, but show a form first, then
+    create the participation.
+
+    Beware that request.user might be anonymous. You should only act on participants acquired
+    using `get_participant`.
+    """
+
     shift: Shift = ...
     method: "BaseSignupMethod" = ...
 
+    def setup(self, request, *args, **kwargs):
+        super().setup(request, *args, **kwargs)
+        self.participant: AbstractParticipant = kwargs["participant"]
+        prevent_getting_participant_from_request_user(request)
+
     def dispatch(self, request, *args, **kwargs):
         if (choice := request.POST.get("signup_choice")) is not None:
             if choice == "sign_up":
@@ -114,7 +177,7 @@ class BaseSignupView(View):
     def signup_pressed(self, **signup_kwargs):
         try:
             with transaction.atomic():
-                self.method.perform_signup(self.request.user.as_participant(), **signup_kwargs)
+                self.method.perform_signup(self.participant, **signup_kwargs)
                 messages.success(
                     self.request,
                     self.method.signup_success_message.format(shift=self.shift),
@@ -122,19 +185,19 @@ class BaseSignupView(View):
         except ParticipationError as errors:
             for error in errors:
                 messages.error(self.request, self.method.signup_error_message.format(error=error))
-        return redirect(self.shift.event.get_absolute_url())
+        return redirect(self.participant.reverse_event_detail(self.shift.event))
 
     def decline_pressed(self, **decline_kwargs):
         try:
             with transaction.atomic():
-                self.method.perform_decline(self.request.user.as_participant(), **decline_kwargs)
+                self.method.perform_decline(self.participant, **decline_kwargs)
                 messages.info(
                     self.request, self.method.decline_success_message.format(shift=self.shift)
                 )
         except ParticipationError as errors:
             for error in errors:
                 messages.error(self.request, self.method.decline_error_message.format(error=error))
-        return redirect(self.shift.event.get_absolute_url())
+        return redirect(self.participant.reverse_event_detail(self.shift.event))
 
 
 def check_event_is_active(method, participant):
@@ -198,6 +261,19 @@ def check_participant_age(method, participant):
         )
 
 
+def get_conflicting_participations(shift, participant):
+    return participant.all_participations().filter(
+        ~Q(shift=shift)
+        & Q(state=AbstractParticipation.States.CONFIRMED)
+        & Q(shift__start_time__lt=shift.end_time, shift__end_time__gt=shift.start_time)
+    )
+
+
+def check_conflicting_shifts(method, participant):
+    if get_conflicting_participations(method.shift, participant).exists():
+        return ParticipationError(_("You are already confirmed for another shift at this time."))
+
+
 class BaseSignupMethod:
     @property
     def slug(self):
@@ -239,16 +315,17 @@ class BaseSignupMethod:
         return self.signup_view_class.as_view(method=self, shift=self.shift)
 
     @property
-    def signup_checkers(self):
+    def _signup_checkers(self):
         return [
             check_event_is_active,
             check_participation_state_for_signup,
             check_inside_signup_timeframe,
             check_participant_age,
+            check_conflicting_shifts,
         ]
 
     @property
-    def decline_checkers(self):
+    def _decline_checkers(self):
         return [
             check_event_is_active,
             check_participation_state_for_decline,
@@ -259,7 +336,7 @@ class BaseSignupMethod:
     def get_signup_errors(self, participant) -> List[ParticipationError]:
         return [
             error
-            for checker in self.signup_checkers
+            for checker in self._signup_checkers
             if (error := checker(self, participant)) is not None
         ]
 
@@ -267,7 +344,7 @@ class BaseSignupMethod:
     def get_decline_errors(self, participant):
         return [
             error
-            for checker in self.decline_checkers
+            for checker in self._decline_checkers
             if (error := checker(self, participant)) is not None
         ]
 
@@ -284,12 +361,19 @@ class BaseSignupMethod:
 
     def perform_signup(self, participant: AbstractParticipant, **kwargs) -> AbstractParticipation:
         """
-        Configure a participation object for the given participant according to the method's configuration.
-        `kwargs` may contain further instructions from e.g. a form.
+        Creates and/or configures a participation object for a given participant and sends out notifications.
+        Passes the participation and kwargs to configure_participation to do configuration specific to the signup method
         """
+        from ephios.core.notifications.types import ResponsibleParticipationRequested
+
         if errors := self.get_signup_errors(participant):
             raise ParticipationError(errors)
-        return self.get_participation_for(participant)
+        participation = self._configure_participation(
+            self.get_participation_for(participant), **kwargs
+        )
+        participation.save()
+        ResponsibleParticipationRequested.send(participation)
+        return participation
 
     def perform_decline(self, participant, **kwargs):
         """Create and configure a declining participation object for the given participant. `kwargs` may contain further instructions from a e.g. a form."""
@@ -300,44 +384,84 @@ class BaseSignupMethod:
         participation.save()
         return participation
 
+    def _configure_participation(
+        self, participation: AbstractParticipation, **kwargs
+    ) -> AbstractParticipation:
+        """
+        Configure the given participation object for signup according to the method's configuration.
+        You need at least to set the participations state. `kwargs` may contain further instructions from e.g. a form.
+        """
+        return NotImplemented
+
     def get_configuration_fields(self):
-        return {
-            "minimum_age": {
-                "formfield": forms.IntegerField(required=False, min_value=1, max_value=999),
-                "default": 16,
-                "publish_with_label": _("Minimum age"),
-            },
-            "signup_until": {
-                "formfield": forms.SplitDateTimeField(
-                    required=False, widget=CustomSplitDateTimeWidget
-                ),
-                "default": None,
-                "publish_with_label": _("Signup until"),
-                "format": functools.partial(formats.date_format, format="SHORT_DATETIME_FORMAT"),
-            },
-            "user_can_decline_confirmed": {
-                "formfield": forms.BooleanField(
-                    label=_("Confirmed users can decline by themselves"),
-                    required=False,
-                    help_text=_("only if the signup timeframe has not ended"),
-                ),
-                "default": False,
-                "publish_with_label": _("Can decline after confirmation"),
-                "format": yesno,
-            },
-        }
+        return OrderedDict(
+            {
+                "minimum_age": {
+                    "formfield": forms.IntegerField(required=False, min_value=1, max_value=999),
+                    "default": 16,
+                    "publish_with_label": _("Minimum age"),
+                },
+                "signup_until": {
+                    "formfield": forms.SplitDateTimeField(
+                        required=False, widget=CustomSplitDateTimeWidget
+                    ),
+                    "default": None,
+                    "publish_with_label": _("Signup until"),
+                    "format": functools.partial(
+                        formats.date_format, format="SHORT_DATETIME_FORMAT"
+                    ),
+                },
+                "user_can_decline_confirmed": {
+                    "formfield": forms.BooleanField(
+                        label=_("Confirmed users can decline by themselves"),
+                        required=False,
+                        help_text=_("only if the signup timeframe has not ended"),
+                    ),
+                    "default": False,
+                    "publish_with_label": _("Can decline after confirmation"),
+                    "format": yesno,
+                },
+            }
+        )
 
     def get_signup_info(self):
         """
         Return key/value pairs about the configuration to show in the shift info box.
         """
         fields = self.get_configuration_fields()
-        return {
-            label: field.get("format", str)(value)
-            for key, field in fields.items()
-            if (label := field.get("publish_with_label", False))
-            and (value := getattr(self.configuration, key))
-        }
+        return OrderedDict(
+            {
+                label: field.get("format", str)(value)
+                for key, field in fields.items()
+                if (label := field.get("publish_with_label", False))
+                and (value := getattr(self.configuration, key))
+            }
+        )
+
+    def get_participant_count_bounds(self):
+        """
+        Return a typle of min, max for how many participants are allowed for the shift.
+        Use None for any value if it is not specifiable.
+        The default implementation returns None, None to signify both values are not specified.
+        """
+        return None, None
+
+    def get_signup_stats(self):
+        """
+        Return an instance of SignupStats for the shift.
+        """
+        min_count, max_count = self.get_participant_count_bounds()
+        participations = list(self.shift.participations.all())
+        return SignupStats(
+            requested_count=sum(
+                p.state == AbstractParticipation.States.REQUESTED for p in participations
+            ),
+            signed_up_count=sum(
+                p.state == AbstractParticipation.States.CONFIRMED for p in participations
+            ),
+            min_count=min_count,
+            max_count=max_count,
+        )
 
     def render_shift_state(self, request):
         """
@@ -346,6 +470,18 @@ class BaseSignupMethod:
         """
         return ""
 
+    def get_participation_display(self):
+        """
+        Returns a displayable representation of participation that can be rendered into a table (e.g. for pdf export).
+        Must return a list of participations or empty slots. Each element of the list has to be a list of a fixed
+        size where each entry is rendered to a separate column.
+        Ex.: [["participant1_name", "participant1_qualification"], ["participant2_name", "participant2_qualification"]]
+        """
+        return [
+            [f"{participant.first_name} {participant.last_name}"]
+            for participant in self.shift.get_participants()
+        ]
+
     def get_configuration_form(self, *args, **kwargs):
         if self.shift is not None:
             kwargs.setdefault("initial", self.configuration.__dict__)
@@ -357,6 +493,30 @@ class BaseSignupMethod:
     def render_configuration_form(self, *args, form=None, **kwargs):
         form = form or self.get_configuration_form(*args, **kwargs)
         template = Template(
-            template_string="{% load bootstrap4 %}{% bootstrap_form form %}"
+            template_string="{% load crispy_forms_filters %}{{ form|crispy }}"
         ).render(Context({"form": form}))
         return template
+
+
+@dataclasses.dataclass()
+class SignupStats:
+    requested_count: int
+    signed_up_count: int
+    min_count: Optional[int]
+    max_count: Optional[int]
+
+    def __add__(self, other: "SignupStats"):
+        if self.min_count is not None or other.min_count is not None:
+            min_count = (self.min_count or 0) + (other.min_count or 0)
+        else:
+            min_count = None
+        if self.max_count is not None and other.max_count is not None:
+            max_count = (self.max_count or 0) + (other.max_count or 0)
+        else:
+            max_count = None
+        return SignupStats(
+            requested_count=self.requested_count + other.requested_count,
+            signed_up_count=self.signed_up_count + other.signed_up_count,
+            min_count=min_count,
+            max_count=max_count,
+        )
