@@ -1,17 +1,16 @@
+from django.conf import settings
 from django.contrib.auth.password_validation import MinimumLengthValidator
 from django.core.mail import EmailMultiAlternatives
 from django.template.loader import render_to_string
 from django.utils.translation import gettext as _
 
-from ephios.settings import SITE_URL
-
 
 class CustomMinimumLengthValidator(MinimumLengthValidator):
     def password_changed(self, password, user):
         if user is not None:
             text_content = _(
                 "Your password for {site} has been changed. If you didn't request this change, contact an administrator immediately."
-            ).format(site=SITE_URL)
+            ).format(site=settings.SITE_URL)
             html_content = render_to_string("email_base.html", {"message_text": text_content})
             message = EmailMultiAlternatives(
                 to=[user.email],
