@@ -1,10 +1,11 @@
+import functools
 import json
 from datetime import datetime, timedelta
 
 from django.contrib import messages
 from django.contrib.auth.mixins import LoginRequiredMixin
 from django.core.exceptions import ValidationError
-from django.db.models import Max, Min
+from django.db.models import Max, Min, Prefetch
 from django.forms import DateField
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404, redirect
@@ -27,9 +28,16 @@ from django.views.generic.detail import SingleObjectMixin
 from guardian.shortcuts import assign_perm, get_objects_for_user, get_users_with_perms
 from recurrence.forms import RecurrenceField
 
-from ephios.core.forms.events import EventDuplicationForm, EventForm
+from ephios.core.forms.events import EventDuplicationForm, EventForm, EventNotificationForm
 from ephios.core.models import Event, EventType, Shift
-from ephios.extra.permissions import CustomPermissionRequiredMixin, get_groups_with_perms
+from ephios.core.notifications.types import (
+    CustomEventParticipantNotification,
+    EventReminderNotification,
+    NewEventNotification,
+)
+from ephios.core.signals import event_forms
+from ephios.extra.mixins import CanonicalSlugDetailMixin, CustomPermissionRequiredMixin
+from ephios.extra.permissions import get_groups_with_perms
 
 
 class EventListView(LoginRequiredMixin, ListView):
@@ -44,6 +52,8 @@ class EventListView(LoginRequiredMixin, ListView):
             )
             .filter(end_time__gte=timezone.now())
             .select_related("type")
+            .prefetch_related("shifts")
+            .prefetch_related(Prefetch("shifts__participations"))
             .order_by("start_time")
         )
 
@@ -52,7 +62,7 @@ class EventListView(LoginRequiredMixin, ListView):
         return super().get_context_data(**kwargs)
 
 
-class EventDetailView(CustomPermissionRequiredMixin, DetailView):
+class EventDetailView(CustomPermissionRequiredMixin, CanonicalSlugDetailMixin, DetailView):
     model = Event
     permission_required = "core.view_event"
 
@@ -62,7 +72,29 @@ class EventDetailView(CustomPermissionRequiredMixin, DetailView):
         return Event.objects.all()
 
 
-class EventUpdateView(CustomPermissionRequiredMixin, UpdateView):
+class EventEditMixin:
+    @functools.cached_property
+    def plugin_forms(self):
+        forms = []
+        for __, resp in event_forms.send(sender=None, event=self.object, request=self.request):
+            forms.extend(resp)
+        return forms
+
+    def get_context_data(self, **kwargs):
+        kwargs["plugin_forms"] = self.plugin_forms
+        return super().get_context_data(**kwargs)
+
+    def is_valid(self, form):
+        return form.is_valid() and all(plugin_form.is_valid() for plugin_form in self.plugin_forms)
+
+    def form_valid(self, form):
+        response = super().form_valid(form)
+        for plugin_form in self.plugin_forms:
+            plugin_form.save()
+        return response
+
+
+class EventUpdateView(CustomPermissionRequiredMixin, EventEditMixin, UpdateView):
     model = Event
     queryset = Event.all_objects.all()
     permission_required = "core.change_event"
@@ -72,13 +104,27 @@ class EventUpdateView(CustomPermissionRequiredMixin, UpdateView):
             data=self.request.POST or None, user=self.request.user, instance=self.object
         )
 
+    def post(self, request, *args, **kwargs):
+        self.object = self.get_object()
+        form = self.get_form()
+        if self.is_valid(form):
+            return self.form_valid(form)
+        return self.form_invalid(form)
 
-class EventCreateView(CustomPermissionRequiredMixin, CreateView):
+
+class EventCreateView(CustomPermissionRequiredMixin, EventEditMixin, CreateView):
     template_name = "core/event_form.html"
     permission_required = "core.add_event"
     accept_object_perms = False
     model = EventType
 
+    def post(self, request, *args, **kwargs):
+        form = self.get_form()
+        self.object = form.instance
+        if self.is_valid(form):
+            return self.form_valid(form)
+        return self.form_invalid(form)
+
     def dispatch(self, request, *args, **kwargs):
         self.eventtype = get_object_or_404(EventType, pk=self.kwargs.get("type"))
         return super().dispatch(request, *args, **kwargs)
@@ -114,7 +160,7 @@ class EventActivateView(CustomPermissionRequiredMixin, SingleObjectMixin, View):
             )
         except ValidationError as e:
             messages.error(request, e)
-        return redirect(reverse("core:event_detail", kwargs={"pk": event.pk}))
+        return redirect(event.get_absolute_url())
 
 
 class EventDeleteView(CustomPermissionRequiredMixin, DeleteView):
@@ -238,3 +284,25 @@ class RRuleOccurrenceView(CustomPermissionRequiredMixin, View):
 
 class HomeView(LoginRequiredMixin, TemplateView):
     template_name = "core/home.html"
+
+
+class EventNotificationView(CustomPermissionRequiredMixin, SingleObjectMixin, FormView):
+    model = Event
+    permission_required = "core.change_event"
+    template_name = "core/event_notification.html"
+    form_class = EventNotificationForm
+
+    def setup(self, request, *args, **kwargs):
+        super().setup(request, *args, **kwargs)
+        self.object = self.get_object()
+
+    def form_valid(self, form):
+        action = form.cleaned_data["action"]
+        if action == form.NEW_EVENT:
+            NewEventNotification.send(self.object)
+        elif action == form.REMINDER:
+            EventReminderNotification.send(self.object)
+        elif action == form.PARTICIPANTS:
+            CustomEventParticipantNotification.send(self.object, form.cleaned_data["mail_content"])
+        messages.success(self.request, _("Notifications sent succesfully."))
+        return redirect(self.object.get_absolute_url())
