@@ -12,6 +12,7 @@ from django.urls import reverse
 from django.utils.translation import gettext_lazy as _
 from django.views.generic import FormView
 from django_select2.forms import Select2MultipleWidget
+from dynamic_preferences.registries import global_preferences_registry
 
 from ephios.core.models import AbstractParticipation, Qualification
 from ephios.core.signup import (
@@ -152,7 +153,7 @@ class SectionBasedSignupView(FormView, BaseSignupView):
     @cached_property
     def sections_participant_qualifies_for(self):
         return sections_participant_qualifies_for(
-            self.method.configuration.sections, self.request.user.as_participant()
+            self.method.configuration.sections, self.participant
         )
 
     def get_form(self, form_class=None):
@@ -183,13 +184,13 @@ class SectionBasedSignupView(FormView, BaseSignupView):
             # do straight signup if choosing is not enabled
             return super().signup_pressed(**kwargs)
 
-        if not self.method.can_sign_up(self.request.user.as_participant()):
+        if not self.method.can_sign_up(self.participant):
             # redirect a misled request
             messages.warning(self.request, _("You can not sign up for this shift."))
-            return redirect(reverse("core:event_detail", kwargs=dict(pk=self.shift.event_id)))
+            return redirect(self.participant.reverse_event_detail(self.shift.event))
 
         # all good, redirect to the form
-        return redirect(reverse("core:signup_action", kwargs=dict(pk=self.shift.pk)))
+        return redirect(self.participant.reverse_signup_action(self.shift))
 
 
 class SectionBasedSignupMethod(BaseSignupMethod):
@@ -230,26 +231,28 @@ class SectionBasedSignupMethod(BaseSignupMethod):
             },
         }
 
+    def get_participant_count_bounds(self):
+        return sum(section.get("min_count") or 0 for section in self.configuration.sections), None
+
     @staticmethod
     def check_qualification(method, participant):
         if not sections_participant_qualifies_for(method.configuration.sections, participant):
             return ParticipationError(_("You are not qualified."))
 
     @property
-    def signup_checkers(self):
-        return super().signup_checkers + [self.check_qualification]
+    def _signup_checkers(self):
+        return super()._signup_checkers + [self.check_qualification]
 
     # pylint: disable=arguments-differ
-    def perform_signup(
-        self, participant: AbstractParticipant, preferred_section_uuid=None, **kwargs
-    ):
-        participation = super().perform_signup(participant, **kwargs)
+    def _configure_participation(
+        self, participation: AbstractParticipation, preferred_section_uuid=None, **kwargs
+    ) -> AbstractParticipation:
         participation.data["preferred_section_uuid"] = preferred_section_uuid
         if preferred_section_uuid:
             # reset dispatch decision, as that would have overwritten the preferred choice
             participation.data["dispatched_section_uuid"] = None
         participation.state = AbstractParticipation.States.REQUESTED
-        participation.save()
+        return participation
 
     def render_configuration_form(self, *args, form=None, **kwargs):
         form = form or self.get_configuration_form(*args, **kwargs)
@@ -258,13 +261,21 @@ class SectionBasedSignupMethod(BaseSignupMethod):
         )
         return template
 
-    def render_shift_state(self, request):
+    def _get_sections_with_users(self):
+        relevant_qualification_categories = global_preferences_registry.manager()[
+            "general__relevant_qualification_categories"
+        ]
         section_by_uuid = {section["uuid"]: section for section in self.configuration.sections}
         # get name and preferred section uuid for confirmed participants
         # if they have a section assigned and we have that section on record
         confirmed_participations = [
             {
                 "name": str(participation.participant),
+                "relevant_qualifications": ", ".join(
+                    participation.participant.qualifications.filter(
+                        category__in=relevant_qualification_categories
+                    ).values_list("abbreviation", flat=True)
+                ),
                 "uuid": dispatched_section_uuid,
             }
             for participation in self.shift.participations.filter(
@@ -274,20 +285,27 @@ class SectionBasedSignupMethod(BaseSignupMethod):
             and dispatched_section_uuid in section_by_uuid
         ]
         # group by section and do some stats
-        confirmed_sections_with_users = [
-            (section_by_uuid.get(uuid), [user["name"] for user in group])
+        sections_with_users = [
+            (
+                section_by_uuid.pop(uuid),
+                [[user["name"], user["relevant_qualifications"]] for user in group],
+            )
             for uuid, group in groupby(
                 sorted(confirmed_participations, key=itemgetter("uuid")), itemgetter("uuid")
             )
         ]
+        # add sections without participants
+        sections_with_users += [(section, None) for section in section_by_uuid.values()]
+        return sections_with_users
 
+    def render_shift_state(self, request):
         return get_template("basesignup/section_based/fragment_state.html").render(
             {
                 "shift": self.shift,
                 "requested_participations": (
                     self.shift.participations.filter(state=AbstractParticipation.States.REQUESTED)
                 ),
-                "confirmed_sections_with_users": confirmed_sections_with_users,
+                "sections_with_users": self._get_sections_with_users(),
                 "disposition_url": (
                     reverse(
                         "core:shift_disposition",
@@ -298,3 +316,20 @@ class SectionBasedSignupMethod(BaseSignupMethod):
                 ),
             }
         )
+
+    def get_participation_display(self):
+        confirmed_sections_with_users = self._get_sections_with_users()
+        participation_display = []
+        for section, users in confirmed_sections_with_users:
+            if users:
+                participation_display += [[user[0], user[1], section["title"]] for user in users]
+            if not users or len(users) < section["min_count"]:
+                required_qualifications = ", ".join(
+                    Qualification.objects.filter(pk__in=section["qualifications"]).values_list(
+                        "abbreviation", flat=True
+                    )
+                )
+                participation_display += [["", required_qualifications, section["title"]]] * (
+                    section["min_count"] - (len(users) if users else 0)
+                )
+        return participation_display
