@@ -1,12 +1,17 @@
-from django.core.exceptions import PermissionDenied
-from django.views.generic import DetailView
+from django.contrib.auth.views import redirect_to_login
+from django.views.generic import DetailView, ListView
 
+from ephios.core.views.settings import SettingsViewMixin
+from ephios.extra.mixins import StaffRequiredMixin
 from ephios.plugins.pages.models import Page
 
 
+class PageListView(StaffRequiredMixin, SettingsViewMixin, ListView):
+    model = Page
+
+
 class PageView(DetailView):
     model = Page
-    template_name = "pages/view_page.html"
 
     def setup(self, request, *args, **kwargs):
         super().setup(request, *args, **kwargs)
@@ -14,5 +19,5 @@ class PageView(DetailView):
 
     def dispatch(self, request, *args, **kwargs):
         if not request.user.is_authenticated and not self.object.publicly_visible:
-            raise PermissionDenied
+            return redirect_to_login(self.request.get_full_path())
         return super().dispatch(request, *args, **kwargs)
